/*
 * This light shader is designed to be used with the shadow_surface shader
 * to compute shadows.
 *
 * Portions (c) Copyright 1999-2003 Okan Arikan
 * Portions (c) 2007 Jeremy Huddleston <jeremyhu@cs.berkeley.edu>
 *
 * Released under the terms of the LGPL-2:
 * http://www.fsf.org/licensing/licenses/lgpl.txt
 */

light shadow_point_grouped(float blur = 0.0;
                           uniform float samples = 16;
                           string shadowname_px = "";
                           string shadowname_nx = "";
                           string shadowname_py = "";
                           string shadowname_ny = "";
                           string shadowname_pz = "";
                           string shadowname_nz = "";
                           
                           point from = point "shader" (0,0,0);
                           uniform float group_id = 0;) {

    float ax,ay,az;
    point Lworld;

    if (shadowname_px == "" || shadowname_nx == "" ||
        shadowname_py == "" || shadowname_ny == "" ||
        shadowname_pz == "" || shadowname_nz == "") {

        illuminate (from) {
            Cl = shadow("raytrace", Ps, "blur", blur, "samples", samples);
        }

    } else {
        illuminate (from) {
            Lworld = transform("world", L + point "world" (0,0,0));
            
            ax = abs(xcomp(Lworld));
            ay = abs(ycomp(Lworld));
            az = abs(zcomp(Lworld));
            
            if ((ax > ay) && (ax > az)) {
                if (xcomp(Lworld) > 0.0)    Cl = shadow( shadowname_px, Ps, "blur", blur, "samples", samples);
                else                        Cl = shadow( shadowname_nx, Ps, "blur", blur, "samples", samples);
            } else if ((ay > ax) && (ay > az)) {
                if (ycomp(Lworld) > 0.0)    Cl = shadow( shadowname_py, Ps, "blur", blur, "samples", samples);
                else                        Cl = shadow( shadowname_ny, Ps, "blur", blur, "samples", samples);
            } else if ((az > ay) && (az > ax)) {
                if (zcomp(Lworld) > 0.0)    Cl = shadow( shadowname_pz, Ps, "blur", blur, "samples", samples);
                else                        Cl = shadow( shadowname_nz, Ps, "blur", blur, "samples", samples);
            }
        }
    }
}
