/*
** Copyright (c) 2001 PIXAR.  All rights reserved.  This program or
** documentation contains proprietary confidential information and trade
** secrets of PIXAR.  Reverse engineering of object code is prohibited.
** Use of copyright notice is precautionary and does not imply
** publication.
**
**                      RESTRICTED RIGHTS NOTICE
**
** Use, duplication, or disclosure by the Government is subject to the
** following restrictions:  For civilian agencies, subparagraphs (a) through
** (d) of the Commercial Computer Software--Restricted Rights clause at
** 52.227-19 of the FAR; and, for units of the Department of Defense, DoD
** Supplement to the FAR, clause 52.227-7013 (c)(1)(ii), Rights in
** Technical Data and Computer Software.
**
** Pixar
** 1200 Park Ave
** Emeryville, CA 94608
**
** ----------------------------------------------------------------------------
*/

/* code generated by slim 6.5.3, RAT 6.5.3 (May 15 2006 03:07:00) */
/* for jeremy@yuffie.outersquare.org at: Fri Apr 20 16:23:01 PDT 2007 */
/* shader modification timestamp: 1177110230 */

/* includes ---------------------------------*/
#include "pxslShading.h"
#include "pxslAOV.h"
#include "pxslRayUtil.h"
/* defines ----------------------------------*/
#define SLIM_TYPEID_surface 0
#define SLIM_TYPEID_displacement 1
#define SLIM_TYPEID_volume 2
#define SLIM_TYPEID_light 3
#define SLIM_VERSION 650
#define SLIM_SHADERTYPE surface
#define SLIM_SHADERTYPEID SLIM_TYPEID_surface
#define SLIM_INSTANCETYPE shadingmodel_aov
#define SLIM_INSTANCENAME Plastic

/* shader body ------------------------------*/
SLIM_SHADERTYPE
SLIM_INSTANCENAME (
  output varying color _ambient=0;
  output varying color _backscattering=0;
  output varying color _diffuse=0;
  output varying color _incandescence=0;
  output varying color _indirect=0;
  output varying color _reflection=0;
  output varying color _refraction=0;
  output varying color _rim=0;
  output varying color _shadow=0;
  output varying color _specular=0;
  output varying color _subsurfacescattering=0;
  output varying color _surfacecolor=0;
  output varying color _thintranslucence=0;
)
{

/* static functions ----------------------*/

		void pxslSpecularSC(
		    float Ks;
		    color coloration;
		    float roughness;
		    uniform float lightType;
		    normal Ns;
		    vector V;
		    output color col;
		    DECLARE_AOV_OUTPUT_PARAMS
		)
		{
		    INIT_AOV_OUTPUT_PARAMS

		    col = 0;
#if SLIM_SHADERTYPEID != SLIM_TYPEID_light
		    extern point P;
		    if (lightType <= 0)
		    {
			illuminance("-environment", P, Ns, 1.57,
			    "lightcache", "reuse")
			{
			    float nonspec = 0;
			    lightsource("__nonspecular", nonspec);
			    if (nonspec < 1)
				col += Cl * (1-nonspec) * 
				    specularbrdf(normalize(L), Ns, 
				    V, roughness);
			}
		    }
		    if (lightType >= 0)
		    {
			float cone;
			cone = 1.55 * pow(roughness,2);
			illuminance ("environment", P, Ns, 1.57,
			    "send:light:__coneangle", cone,
			    "send:light:__coneaxis", reflect(-V, Ns),
			    "lightcache", "refresh")
			{
			    float nonspec = 0;
			    lightsource("__nonspecular", nonspec);
			    if (nonspec < 1)
				col += (1-nonspec) * Cl;
			}
		    }
		    col *= coloration * Ks;
		    c_specular = col;
#endif
		}
	    

	void
	pxslSurfaceNormal(uniform float normalize;
			output normal n;)
	{
	    extern normal N;
	    if(normalize == 0)
		n = N;
	    else
		n = normalize(N);
	}
    

	void
	pxslIncidentDirection(uniform float normalize;
			    uniform float flip;
			    output vector V;)
	{
	    extern vector I;
	    if(normalize == 0)
		V = I;
	    else
		V = normalize(I);
	    if(flip != 0)
		V = -V;
	}
    

	void
	pxslViewDirection(  uniform float normalize;
			    uniform float flip;
			    output vector V;)
	{
	    extern vector I;
	    V = -I;
	    if(normalize != 0)
		V = normalize(V);
	    if(flip != 0)
		V = -V;
	}
    

		void pxslDiffuseSC(
		    float Kd;
		    color coloration;
		    uniform float lightType;
		    normal Ns;
		    output color col;
		    DECLARE_AOV_OUTPUT_PARAMS
		)
		{
		    INIT_AOV_OUTPUT_PARAMS

		    col = 0;
#if SLIM_SHADERTYPEID != SLIM_TYPEID_light
		    extern point P;
		    color inshadow;

		    if (lightType <= 0)
		    {
			illuminance("-environment", P, Ns, 1.57,
			    "lightcache", "reuse")
			{
			    float nondiff = 0;
			    lightsource("__nondiffuse", nondiff);
			    if (nondiff < 1)
			    {
				if( 0 != lightsource("__inShadowC", inshadow) )
				    c_shadow += inshadow;
				col += Cl * (1-nondiff) * 
				    normalize(L).Ns;
			    }
			}
		    }
		    if (lightType >= 0)
		    {
			illuminance ("environment", P, Ns, 1.57,
			    "send:light:__coneangle", 1.57,
			    "send:light:__coneaxis", Ns,
			    "lightcache", "refresh")
			{
			    float nondiff = 0;
			    lightsource("__nondiffuse", nondiff);
			    if (nondiff < 1)
			    {
				if( 0 != lightsource("__inShadowC", inshadow) )
				    c_shadow += inshadow;
				col += (1-nondiff) * Cl;
			    }
			}
		    }
		    col *= coloration * Kd;
#endif
		    c_surfacecolor = coloration;
		    c_diffuse = col;
		}
	    

	void
	pxslShadingNormal(varying vector incident;
			  varying normal n;
			  output normal ns;)
	{
	    uniform float depth;
	    rayinfo("depth", depth);
	    if(depth > 0)
		ns = faceforward(n, incident, n);
	    else
	    {
		uniform float sides = 2;
		attribute("Sides", sides);
		if(sides == 2)
		    ns = faceforward(n, incident, n);
		else
		    ns = n;
	    }
	}
    

/* dynamic functions ------------------------*/
    
/* ----------------------------------------------*/
    void pxslPlastic_func3 ( /* Plastic */
      uniform color SurfaceOpacity;
      uniform float Ka;
      uniform color AmbColor;
      color diff_col;
      color diff_shadow;
      color diff_surfacecolor;
      color diff_incandescence;
      color diff_ambient;
      color diff_indirect;
      color diff_diffuse;
      color diff_thintranslucence;
      color diff_subsurfacescattering;
      color diff_backscattering;
      color diff_specular;
      color diff_rim;
      color diff_reflection;
      color diff_refraction;
      color spec_col;
      color spec_shadow;
      color spec_surfacecolor;
      color spec_incandescence;
      color spec_ambient;
      color spec_indirect;
      color spec_diffuse;
      color spec_thintranslucence;
      color spec_subsurfacescattering;
      color spec_backscattering;
      color spec_specular;
      color spec_rim;
      color spec_reflection;
      color spec_refraction;
      output color CI;
      output color OI;
      output color c_shadow;
      output color c_surfacecolor;
      output color c_incandescence;
      output color c_ambient;
      output color c_indirect;
      output color c_diffuse;
      output color c_thintranslucence;
      output color c_subsurfacescattering;
      output color c_backscattering;
      output color c_specular;
      output color c_rim;
      output color c_reflection;
      output color c_refraction;
      )
    {
      color amb;
      OI = SurfaceOpacity;
      amb = AmbColor * Ka * ambient();
      CI = amb + diff_col + spec_col;
      CI *= SurfaceOpacity;
      c_surfacecolor = SurfaceOpacity * diff_surfacecolor + spec_surfacecolor;
      c_incandescence = SurfaceOpacity * diff_incandescence + spec_incandescence;
      c_ambient = SurfaceOpacity * diff_ambient + spec_ambient;
      c_indirect = SurfaceOpacity * diff_indirect + spec_indirect;
      c_diffuse = SurfaceOpacity * diff_diffuse + spec_diffuse;
      c_thintranslucence = SurfaceOpacity * diff_thintranslucence + spec_thintranslucence;
      c_subsurfacescattering = SurfaceOpacity * diff_subsurfacescattering + spec_subsurfacescattering;
      c_backscattering = SurfaceOpacity * diff_backscattering + spec_backscattering;
      c_specular = SurfaceOpacity * diff_specular + spec_specular;
      c_rim = SurfaceOpacity * diff_rim + spec_rim;
      c_reflection = SurfaceOpacity * diff_reflection + spec_reflection;
      c_refraction = SurfaceOpacity * diff_refraction + spec_refraction;
      c_ambient += SurfaceOpacity * amb;
      c_shadow = (diff_shadow + spec_shadow);
    }

/* local variables --------------------------*/
  normal tmp2;
  normal tmp3;
  vector tmp0;
  vector tmp1;
  color tmp29;
  color tmp22;
  color tmp20;
  color tmp44;
  color tmp37;
  color tmp9;
  color tmp23;
  color tmp40;
  color tmp32;
  color tmp21;
  color tmp4;
  color tmp45;
  color tmp36;
  color tmp46;
  color tmp42;
  color tmp24;
  color tmp39;
  color tmp34;
  color tmp15;
  color tmp8;
  color tmp43;
  color tmp41;
  color tmp35;
  color tmp11;
  color tmp25;
  color tmp16;
  color tmp7;
  color tmp38;
  color tmp30;
  color tmp18;
  color tmp17;
  color tmp13;
  color tmp31;
  color tmp27;
  color tmp5;
  color tmp10;
  color tmp19;
  color tmp14;
  color tmp12;
  color tmp33;
  color tmp28;
  color tmp26;
  color tmp6;

/* main body --------------------------------*/
  /* generate (all) ----------- */
  pxslIncidentDirection ( /* IncidentDirection */
    1, /* Normalize */
    0, /* Flip */
    tmp0  /* result */ 
    );
  pxslViewDirection ( /* ViewDirection */
    1, /* Normalize */
    0, /* Flip */
    tmp1  /* result */ 
    );
  pxslSurfaceNormal ( /* SurfaceNormal */
    1, /* Normalize */
    tmp2  /* result */ 
    );
  pxslShadingNormal ( /* ShadingNormal */
    tmp0, /* Forward */ 
    tmp2, /* Normal */ 
    tmp3  /* result */ 
    );
  pxslDiffuseSC ( /* Diffuse */
    1, /* Intensity */
    color(0.23,0.07,1), /* Color */
    0, /* Use */
    tmp3, /* Normal */ 
    tmp4, /* col */ 
    tmp5, /* c_shadow */ 
    tmp6, /* c_surfacecolor */ 
    tmp7, /* c_incandescence */ 
    tmp8, /* c_ambient */ 
    tmp9, /* c_indirect */ 
    tmp10, /* c_diffuse */ 
    tmp11, /* c_thintranslucence */ 
    tmp12, /* c_subsurfacescattering */ 
    tmp13, /* c_backscattering */ 
    tmp14, /* c_specular */ 
    tmp15, /* c_rim */ 
    tmp16, /* c_reflection */ 
    tmp17  /* c_refraction */ 
    );
  pxslSpecularSC ( /* Specular */
    1, /* Intensity */
    color(1,1,1), /* Color */
    .1, /* Roughness */
    0, /* Use */
    tmp3, /* Normal */ 
    tmp1, /* ViewDirection */ 
    tmp18, /* col */ 
    tmp19, /* c_shadow */ 
    tmp20, /* c_surfacecolor */ 
    tmp21, /* c_incandescence */ 
    tmp22, /* c_ambient */ 
    tmp23, /* c_indirect */ 
    tmp24, /* c_diffuse */ 
    tmp25, /* c_thintranslucence */ 
    tmp26, /* c_subsurfacescattering */ 
    tmp27, /* c_backscattering */ 
    tmp28, /* c_specular */ 
    tmp29, /* c_rim */ 
    tmp30, /* c_reflection */ 
    tmp31  /* c_refraction */ 
    );
  pxslPlastic_func3 ( /* Plastic */
    color(1,1,1), /* Opacity */
    .05, /* Ka */
    color(1,1,1), /* Ambient Coloration */
    tmp4, /* diff_col */ 
    tmp5, /* diff_shadow */ 
    tmp6, /* diff_surfacecolor */ 
    tmp7, /* diff_incandescence */ 
    tmp8, /* diff_ambient */ 
    tmp9, /* diff_indirect */ 
    tmp10, /* diff_diffuse */ 
    tmp11, /* diff_thintranslucence */ 
    tmp12, /* diff_subsurfacescattering */ 
    tmp13, /* diff_backscattering */ 
    tmp14, /* diff_specular */ 
    tmp15, /* diff_rim */ 
    tmp16, /* diff_reflection */ 
    tmp17, /* diff_refraction */ 
    tmp18, /* spec_col */ 
    tmp19, /* spec_shadow */ 
    tmp20, /* spec_surfacecolor */ 
    tmp21, /* spec_incandescence */ 
    tmp22, /* spec_ambient */ 
    tmp23, /* spec_indirect */ 
    tmp24, /* spec_diffuse */ 
    tmp25, /* spec_thintranslucence */ 
    tmp26, /* spec_subsurfacescattering */ 
    tmp27, /* spec_backscattering */ 
    tmp28, /* spec_specular */ 
    tmp29, /* spec_rim */ 
    tmp30, /* spec_reflection */ 
    tmp31, /* spec_refraction */ 
    tmp32, /* CI */ 
    tmp33, /* OI */ 
    tmp34, /* c_shadow */ 
    tmp35, /* c_surfacecolor */ 
    tmp36, /* c_incandescence */ 
    tmp37, /* c_ambient */ 
    tmp38, /* c_indirect */ 
    tmp39, /* c_diffuse */ 
    tmp40, /* c_thintranslucence */ 
    tmp41, /* c_subsurfacescattering */ 
    tmp42, /* c_backscattering */ 
    tmp43, /* c_specular */ 
    tmp44, /* c_rim */ 
    tmp45, /* c_reflection */ 
    tmp46  /* c_refraction */ 
    );
  Ci = tmp32;
  Oi = tmp33;
  _shadow = tmp34;
  _surfacecolor = tmp35;
  _incandescence = tmp36;
  _ambient = tmp37;
  _indirect = tmp38;
  _diffuse = tmp39;
  _thintranslucence = tmp40;
  _subsurfacescattering = tmp41;
  _backscattering = tmp42;
  _specular = tmp43;
  _rim = tmp44;
  _reflection = tmp45;
  _refraction = tmp46;
}

